{
TGMMarker component

  ES: componente para mostrar ventanas de informacin en un mapa de Google Maps
      mediante el componente TGMMap
  EN: component to show balloons with information on Google Map map using the
      component TGMMap

=========================================================================
MODO DE USO/HOW TO USE

  ES: poner el componente en el formulario, linkarlo a un TGMMap y poner el
      texto a mostrar en la propiedad "HTMLContent"
  EN: put the component into a form, link to a TGMMap and put the text to show
      into "HTMLContent" property
=========================================================================
History:

ver 0.1:
  ES- primera versin
  EN- first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2012, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMInfoWindow;

interface

uses
  Classes,
  GMLatLng, GMMap, GMBaseInfoWindow;

type
  TGMInfoWindow = class;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TInfoWindow = class(TLinkedComponent)
  private
    FPosition: TLatLng;
    function GetContent: string;
    function GetDisableAutoPan: Boolean;
    function GetMaxWidth: Integer;
    function GetPixelOffset: TIWSize;
    procedure SetContent(const Value: string);
    procedure SetDisableAutoPan(const Value: Boolean);
    procedure SetMaxWidth(const Value: Integer);
    procedure SetPixelOffset(const Value: TIWSize);
    procedure SetCloseOtherBeforeOpen(const Value: Boolean);
    function GetCloseOtherBeforeOpen: Boolean;
  protected
    function GetDisplayName: String; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;
    procedure CenterMapTo;
    procedure OpenClose;
  published
    property HTMLContent: string read GetContent write SetContent;
    property DisableAutoPan: Boolean read GetDisableAutoPan write SetDisableAutoPan;
    property MaxWidth: Integer read GetMaxWidth write SetMaxWidth;  // 0 = no MaxWidth
    property PixelOffset: TIWSize read GetPixelOffset write SetPixelOffset; // (0,0) = no pixelOffset
    property CloseOtherBeforeOpen: Boolean read GetCloseOtherBeforeOpen write SetCloseOtherBeforeOpen;
    property Position: TLatLng read FPosition write FPosition;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TInfoWindows = class(TLinkedComponents)
  private
    function GetItems(I: Integer): TInfoWindow;
    procedure SetItems(I: Integer; const Value: TInfoWindow);
  protected
    function GetOwner: TPersistent; override;
  public
    function Add: TInfoWindow;
    function Insert(Index: Integer): TInfoWindow;

    property Items[I: Integer]: TInfoWindow read GetItems write SetItems; default;
  end;

  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference#InfoWindow
  **************************************************************************** }
  TGMInfoWindow = class(TGMLinkedComponent)
  private
  protected
    function GetAPIUrl: string; override;

    function GetItems(I: Integer): TInfoWindow;

    procedure ShowElements; override;
    function GetCollectionItemClass: TCollectionItemClass; override;
  public
    constructor Create(aOwner: TComponent); override;

    function Add(Lat: Real = 0; Lng: Real = 0; HTMLContent: string = ''): TInfoWindow;

    property Items[I: Integer]: TInfoWindow read GetItems; default;
  published
    property VisualObjects;
  end;

implementation

uses
  SysUtils;

{ TGMInfoWindow }

function TGMInfoWindow.Add(Lat, Lng: Real; HTMLContent: string): TInfoWindow;
begin
  Result := TInfoWindow(inherited Add);
  Result.Position.Lat := Lat;
  Result.Position.Lng := Lng;
  Result.HTMLContent := HTMLContent;
end;

constructor TGMInfoWindow.Create(aOwner: TComponent);
begin
  inherited;

  VisualObjects := TInfoWindows.Create(Self, TInfoWindow);
end;

function TGMInfoWindow.GetAPIUrl: string;
begin
  Result := 'https://developers.google.com/maps/documentation/javascript/reference#InfoWindow';
end;

function TGMInfoWindow.GetCollectionItemClass: TCollectionItemClass;
begin
  Result := TInfoWindow;
end;

function TGMInfoWindow.GetItems(I: Integer): TInfoWindow;
begin
  Result := TInfoWindow(inherited Items[i]);
end;

procedure TGMInfoWindow.ShowElements;
const
  StrParams = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s';
var
  Params: String;
  i: Integer;
begin
  if not ExecuteScript('DeleteObjects', IntToStr(IdxList)) then Exit;

  for i:= 0 to VisualObjects.Count - 1 do
  begin
    Params := Format(StrParams, [
                    QuotedStr(Items[i].InfoWindow.GetConvertedString),
                    LowerCase(BoolToStr(Items[i].DisableAutoPan, True)),
                    IntToStr(Items[i].MaxWidth),
                    IntToStr(Items[i].PixelOffset.Height),
                    IntToStr(Items[i].PixelOffset.Width),
                    Items[i].Position.LatToStr(Map.Precision),
                    Items[i].Position.LngToStr(Map.Precision),
                    LowerCase(BoolToStr(Items[i].CloseOtherBeforeOpen, True)),
                    IntToStr(IdxList),
                    IntToStr(Items[i].Index),
                    LowerCase(BoolToStr(False, True))
                    ]);

    ExecuteScript('InfoWindowsCreate', Params);
  end;
end;

{ TInfoWindows }

function TInfoWindows.Add: TInfoWindow;
begin
  Result := TInfoWindow(inherited Add);
end;

function TInfoWindows.GetItems(I: Integer): TInfoWindow;
begin
  Result := TInfoWindow(inherited Items[I]);
end;

function TInfoWindows.GetOwner: TPersistent;
begin
  Result := TGMInfoWindow(inherited GetOwner);
end;

function TInfoWindows.Insert(Index: Integer): TInfoWindow;
begin
  Result := TInfoWindow(inherited Insert(Index));
end;

procedure TInfoWindows.SetItems(I: Integer; const Value: TInfoWindow);
begin
  inherited SetItem(I, Value);
end;

{ TInfoWindow }

procedure TInfoWindow.Assign(Source: TPersistent);
begin
  if Source is TInfoWindow then
  begin
    InfoWindow.Assign(TInfoWindow(Source).InfoWindow);
    Position.Assign(TInfoWindow(Source).Position);
  end
  else
    inherited Assign(Source);
end;

procedure TInfoWindow.CenterMapTo;
begin
  if Assigned(Collection) and (Collection is TInfoWindows) and
     Assigned(TInfoWindows(Collection).FGMLinkedComponent) and
     Assigned(TInfoWindows(Collection).FGMLinkedComponent.Map) then
    TInfoWindows(Collection).FGMLinkedComponent.Map.SetCenter(Position.Lat, Position.Lng);
end;

constructor TInfoWindow.Create(Collection: TCollection);
begin
  inherited;

  FPosition := TLatLng.Create;
end;

destructor TInfoWindow.Destroy;
begin
  if Assigned(FPosition) then FreeAndNil(FPosition);

  inherited;
end;

function TInfoWindow.GetCloseOtherBeforeOpen: Boolean;
begin
  Result := InfoWindow.CloseOtherBeforeOpen;
end;

function TInfoWindow.GetContent: string;
begin
  Result := InfoWindow.HTMLContent;
end;

function TInfoWindow.GetDisableAutoPan: Boolean;
begin
  Result := InfoWindow.DisableAutoPan;
end;

function TInfoWindow.GetDisplayName: String;
begin
  if Length(InfoWindow.HTMLContent) > 0 then
  begin
    if Length(InfoWindow.HTMLContent) > 15 then
      Result := Copy(InfoWindow.HTMLContent, 0, 12) + '...'
    else
      Result := InfoWindow.HTMLContent;
  end
  else
    Result := inherited GetDisplayName;
end;

function TInfoWindow.GetMaxWidth: Integer;
begin
  Result := InfoWindow.MaxWidth;
end;

function TInfoWindow.GetPixelOffset: TIWSize;
begin
  Result := InfoWindow.PixelOffset;
end;

procedure TInfoWindow.OpenClose;
const
  StrParams = '%s,%s';
var
  Params: String;
begin
  if Assigned(Collection) and (Collection is TInfoWindows) and
     Assigned(TInfoWindows(Collection).FGMLinkedComponent) and
     TGMInfoWindow(TInfoWindows(Collection).FGMLinkedComponent).AutoUpdate then
  begin
    Params := Format(StrParams, [
                      IntToStr(IdxList),
                      IntToStr(Index)]);
    TGMInfoWindow(TInfoWindows(Collection).FGMLinkedComponent).ExecuteScript('InfoWindowOpenClose', Params);
  end;
end;

procedure TInfoWindow.SetCloseOtherBeforeOpen(const Value: Boolean);
begin
  InfoWindow.CloseOtherBeforeOpen := Value;
end;

procedure TInfoWindow.SetContent(const Value: string);
begin
  InfoWindow.HTMLContent := Value;
end;

procedure TInfoWindow.SetDisableAutoPan(const Value: Boolean);
begin
  InfoWindow.DisableAutoPan := Value;
end;

procedure TInfoWindow.SetMaxWidth(const Value: Integer);
begin
  InfoWindow.MaxWidth := MaxWidth;
end;

procedure TInfoWindow.SetPixelOffset(const Value: TIWSize);
begin
  InfoWindow.PixelOffset := Value;
end;

end.
