{
TLatLng class

  ES: LatLng es un punto en las coordenadas geogrficas, latitud y longitud
  EN: LatLng is a point in geographical coordinates, latitude and longitude

=========================================================================
History:

ver 0.1:
  ES: primera versin
  EN: first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2011, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMLatLng;

interface

uses
  Classes;

type
  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference#LatLng
  **************************************************************************** }
  TLatLng = class(TPersistent)
  private
    FLat: Real;
    FLng: Real;
  protected
    procedure SetLat(const Value: Real); virtual;
    procedure SetLng(const Value: Real); virtual;

    procedure LatLngToStr(LatLong: TLatLng; var Lat, Lng: string; Precision: Integer = 6); overload;
    procedure LatLngToStr(Latitude, Longitude: Real; var Lat, Lng: string; Precision: Integer = 6); overload;
  public
    constructor Create(Lat: Real = 0; Lng: Real = 0); virtual;

    procedure Assign(Source: TPersistent); override;

    function IsEqual(Other: TLatLng): Boolean;
    function ToStr(Precision: Integer = 6): string;
    function ToUrlValue(Precision: Integer = 6): string;
    function LatToStr(Precision: Integer = 6): string;
    function LngToStr(Precision: Integer = 6): string;
  published
    property Lat: Real read FLat write SetLat;
    property Lng: Real read FLng write SetLng;
  end;

implementation

uses
  SysUtils;

{ TLatLng }

procedure TLatLng.Assign(Source: TPersistent);
begin
  if Source is TLatLng then
  begin
    Lat := TLatLng(Source).Lat;
    Lng := TLatLng(Source).Lng;
  end
  else
    inherited Assign(Source);
end;

constructor TLatLng.Create(Lat, Lng: Real);
begin
  FLat := Lat;
  FLng := Lng;
end;

function TLatLng.IsEqual(Other: TLatLng): Boolean;
begin
  Result := (FLat = Other.Lat) and (FLng = Other.Lng);
end;

procedure TLatLng.LatLngToStr(LatLong: TLatLng; var Lat, Lng: string;
  Precision: Integer);
begin
  LatLngToStr(LatLong.Lat, LatLong.Lng, Lat, Lng, Precision);
end;

procedure TLatLng.LatLngToStr(Latitude, Longitude: Real; var Lat, Lng: string;
  Precision: Integer);
var
  i: Integer;
  Prec: string;
begin
  if Precision > 0 then
  begin
    Prec := '0.';
    for i := 1 to Precision do Prec := Prec + '0';
    Lat := StringReplace(FormatFloat(Prec, Latitude), ',', '.', [rfReplaceAll]);
    Lng := StringReplace(FormatFloat(Prec, Longitude), ',', '.', [rfReplaceAll]);
  end
  else
  begin
    Lat := StringReplace(FloatToStr(Latitude), ',', '.', [rfReplaceAll]);
    Lng := StringReplace(FloatToStr(Longitude), ',', '.', [rfReplaceAll]);
  end;
end;

function TLatLng.LatToStr(Precision: Integer): string;
var
  La, Ln: string;
begin
  LatLngToStr(FLat, FLng, La, Ln, Precision);
  Result := La;
end;

function TLatLng.LngToStr(Precision: Integer): string;
var
  La, Ln: string;
begin
  LatLngToStr(FLat, FLng, La, Ln, Precision);
  Result := Ln;
end;

procedure TLatLng.SetLat(const Value: Real);
begin
  FLat := Value;
end;

procedure TLatLng.SetLng(const Value: Real);
begin
  FLng := Value;
end;

function TLatLng.ToStr(Precision: Integer): string;
const
  Str = '(%s, %s)';
var
  La, Ln: string;
begin
  LatLngToStr(FLat, FLng, La, Ln, Precision);
  Result := Format(Str, [La, Ln]);
end;

function TLatLng.ToUrlValue(Precision: Integer): string;
const
  Str = '%s,%s';
var
  La, Ln: string;
begin
  LatLngToStr(FLat, FLng, La, Ln, Precision);
  Result := Format(Str, [La, Ln]);
end;

end.
